@@ -9,6 +9,7 @@ from robot.libraries.BuiltIn import BuiltIn
 from selenium.common.exceptions import NoSuchElementException, InvalidSelectorException
 from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.common.touch_actions import TouchActions
+from appium.webdriver.common.touch_action import TouchAction
 from time import sleep, time
 from robot import utils
 from base64 import b64decode
@@ -109,6 +110,7 @@ class DesktopLibrary(AppiumLibrary):
         | Library | DesktopLibrary | timeout=10 | driver_path="C:/WinAppDriver.exe" | # Sets a new path for the WinAppDriver                               |
         """
         self.winappdriver = WinAppDriver(driver_path)
+        self.current_element = None
         super().__init__(timeout, run_on_failure)
 
     def get_keyword_names(self):
@@ -387,7 +389,8 @@ class DesktopLibrary(AppiumLibrary):
 
         See `introduction` for details about locating elements."""
         self._wait_until_page_contains_element(locator, timeout, error)
-        self.clear_text(locator)
+        self._info("Clear text field '%s'" % locator)
+        self.current_element.clear()
 
     def click_element(self, locator):
         """Click element identified by `locator`.
@@ -414,7 +417,8 @@ class DesktopLibrary(AppiumLibrary):
         Use `Wait For And Mouse Over And Click Element` if this keyword gives issues in the
         application."""
         self._wait_until_page_contains_element(locator, timeout, error)
-        self.click_element(locator)
+        self._info("Clicking element '%s'." % locator)
+        self.current_element.click()
 
     @keyword("Wait For And Input Password")
     def wait_for_and_input_password(self, locator, text, timeout=None, error=None):
@@ -426,7 +430,9 @@ class DesktopLibrary(AppiumLibrary):
 
         The difference between this keyword and `Wait For And Input Text` is that this keyword
         does not log the given password. See `introduction` for details about locating elements."""
-        AppiumCommon.wait_for_and_input_password(self, locator, text, timeout, error)
+        self._wait_until_page_contains_element(locator, timeout, error)
+        self._info("Typing password into text field '%s'" % locator)
+        self.current_element.send_keys(text)
 
     @keyword("Wait For And Input Text")
     def wait_for_and_input_text(self, locator, text, timeout=None, error=None):
@@ -437,7 +443,9 @@ class DesktopLibrary(AppiumLibrary):
         ``error`` can be used to override the default error message.
 
         See `introduction` for details about locating elements."""
-        AppiumCommon.wait_for_and_input_text(self, locator, text, timeout, error)
+        self._wait_until_page_contains_element(locator, timeout, error)
+        self._info("Typing text '%s' into text field '%s'" % (text, locator))
+        self.current_element.send_keys(text)
 
     @keyword("Wait For And Long Press")
     def wait_for_and_long_press(self, locator, duration=10000, timeout=None, error=None):
@@ -448,7 +456,9 @@ class DesktopLibrary(AppiumLibrary):
         ``error`` can be used to override the default error message.
 
         See `introduction` for details about locating elements."""
-        AppiumCommon.wait_for_and_long_press(self, locator, duration, timeout, error)
+        self._wait_until_page_contains_element(locator, timeout, error)
+        actions = TouchAction(self._current_application())
+        actions.press(self.current_element).wait(duration).release().perform()
 
     @keyword("Wait Until Element Contains")
     def wait_until_element_contains(self, locator, text, timeout=None, error=None):
@@ -462,7 +472,8 @@ class DesktopLibrary(AppiumLibrary):
         `Wait Until Page Does Not Contain`
         `Wait Until Page Does Not Contain Element`
         """
-        AppiumCommon.wait_until_element_contains(self, locator, text, timeout, error)
+        self._wait_until_page_contains_element(locator, timeout, error)
+        self.element_should_contain_text(self.current_element, text, error)
 
     @keyword("Wait Until Element Does Not Contain")
     def wait_until_element_does_not_contain(self, locator, text, timeout=None, error=None):
@@ -477,7 +488,17 @@ class DesktopLibrary(AppiumLibrary):
         `Wait Until Page Does Not Contain`
         `Wait Until Page Does Not Contain Element`
         """
-        AppiumCommon.wait_until_element_does_not_contain(self, locator, text, timeout, error)
+        self._wait_until_page_contains_element(locator, timeout, error)
+        self.element_should_not_contain_text(self.current_element, text, error)
+
+    @keyword("Element Should Be Enabled")
+    def element_should_be_enabled(self, locator, loglevel='INFO'):
+        """Verifies that element identified with locator is enabled."""
+        element = self._check_for_cached_element(locator)
+        if not element.is_enabled():
+            raise AssertionError("Element '%s' should be enabled "
+                                 "but did not" % locator)
+        self._info("Element '%s' is enabled ." % locator)
 
     @keyword("Wait Until Element Is Enabled")
     def wait_until_element_is_enabled(self, locator, timeout=None, error=None):
@@ -487,9 +508,18 @@ class DesktopLibrary(AppiumLibrary):
 
         ``error`` can be used to override the default error message.
 
-        See also `Wait Until Element Is Disabled`
-        """
-        AppiumCommon.wait_until_element_is_enabled(self, locator, timeout, error)
+        See also `Wait Until Element Is Disabled`"""
+        self._wait_until_page_contains_element(locator, timeout, error)
+        self.element_should_be_enabled(self.current_element)
+
+    @keyword("Element Should Be Disabled")
+    def element_should_be_disabled(self, locator, loglevel='INFO'):
+        """Verifies that element identified with locator is disabled."""
+        element = self._check_for_cached_element(locator)
+        if element.is_enabled():
+            raise AssertionError("Element '%s' should be disabled "
+                                 "but did not" % locator)
+        self._info("Element '%s' is disabled ." % locator)
 
     @keyword("Wait Until Element Is Disabled")
     def wait_until_element_is_disabled(self, locator, timeout=None, error=None):
@@ -499,17 +529,16 @@ class DesktopLibrary(AppiumLibrary):
 
         ``error`` can be used to override the default error message.
 
-        See also `Wait Until Element Is Disabled`
-        """
-        AppiumCommon.wait_until_element_is_disabled(self, locator, timeout, error)
+        See also `Wait Until Element Is Enabled`"""
+        self._wait_until_page_contains_element(locator, timeout, error)
+        self.element_should_be_disabled(self.current_element)
 
     @keyword("Mouse Over Element")
     def mouse_over_element(self, locator, x_offset=0, y_offset=0):
         """Moves the mouse over the given ``locator``.
 
-        ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate.
-        """
-        element = self._element_find(locator, True, True)
+        ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate."""
+        element = self._check_for_cached_element(locator)
         actions = ActionChains(self._current_application())
         self._move_to_element(actions, element, x_offset, y_offset)
         actions.perform()
@@ -523,10 +552,9 @@ class DesktopLibrary(AppiumLibrary):
 
         ``error`` can be used to override the default error message.
 
-        ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate.
-        """
+        ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate."""
         self._wait_until_page_contains_element(locator, timeout, error)
-        self.mouse_over_element(locator, x_offset, y_offset)
+        self.mouse_over_element(self.current_element, x_offset, y_offset)
 
     @keyword("Mouse Over And Click Element")
     def mouse_over_and_click_element(self, locator, double_click=False, x_offset=0, y_offset=0):
@@ -534,8 +562,7 @@ class DesktopLibrary(AppiumLibrary):
 
         ``double_click`` can be used to click twice.
 
-        ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate.
-        """
+        ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate."""
         self.mouse_over_element(locator, x_offset=x_offset, y_offset=y_offset)
         self.click_a_point(double_click=double_click)
 
@@ -545,8 +572,7 @@ class DesktopLibrary(AppiumLibrary):
 
         ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate.
 
-        See also `Mouse Over And Click Element`
-        """
+        See also `Mouse Over And Click Element`"""
         self.mouse_over_element(locator, x_offset=x_offset, y_offset=y_offset)
         self.context_click_a_point()
 
@@ -561,20 +587,17 @@ class DesktopLibrary(AppiumLibrary):
 
         ``double_click`` can be used to click twice.
 
-        ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate.
-        """
+        ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate."""
         self._wait_until_page_contains_element(locator, timeout, error)
-        self.mouse_over_and_click_element(locator, double_click, x_offset, y_offset)
+        self.mouse_over_and_click_element(self.current_element, double_click, x_offset, y_offset)
 
     @keyword("Mouse Over By Offset")
     def mouse_over_by_offset(self, x_offset=0, y_offset=0):
-        """Moves the mouse from its current location by the given ``x_offset`` and ``y_offset``.
-        """
+        """Moves the mouse from its current location by the given ``x_offset`` and ``y_offset``."""
         actions = ActionChains(self._current_application())
-        actions.move_by_offset(x_offset, y_offset)
         self._info('Moving mouse from current location with an '
                    'offset of (%s,%s).' % (x_offset, y_offset))
-        actions.perform()
+        actions.move_by_offset(x_offset, y_offset).perform()
 
     @keyword("Click A Point")
     def click_a_point(self, x_offset=0, y_offset=0, double_click=False):
@@ -582,8 +605,7 @@ class DesktopLibrary(AppiumLibrary):
 
         ``x_offset`` and ``y_offset`` can be applied to give an offset.
 
-        ``double_click`` can be used to click twice.
-        """
+        ``double_click`` can be used to click twice."""
         actions = ActionChains(self._current_application())
         if x_offset != 0 or y_offset != 0:
             actions.move_by_offset(x_offset, y_offset)
@@ -598,8 +620,7 @@ class DesktopLibrary(AppiumLibrary):
     def context_click_a_point(self, x_offset=0, y_offset=0):
         """Right-clicks the current mouse location.
 
-        ``x_offset`` and ``y_offset`` can be applied to give an offset.
-        """
+        ``x_offset`` and ``y_offset`` can be applied to give an offset."""
         actions = ActionChains(self._current_application())
         if x_offset != 0 or y_offset != 0:
             actions.move_by_offset(x_offset, y_offset)
@@ -616,8 +637,7 @@ class DesktopLibrary(AppiumLibrary):
     @keyword("Drag And Drop By Offset")
     def drag_and_drop_by_offset(self, locator, x_offset=0, y_offset=0):
         """Drags the element found with ``locator`` to the given ``x_offset`` and ``y_offset``
-        coordinates.
-        """
+        coordinates."""
         AppiumCommon.drag_and_drop_by_offset(self, locator, x_offset, y_offset)
 
     @keyword("Send Keys")
@@ -717,8 +737,8 @@ class DesktopLibrary(AppiumLibrary):
 
     @keyword("Select Elements From Menu")
     def select_elements_from_menu(self, *args):
-        """Selects N number of elements in the order they are given. This is useful for working
-        though a nested menu listing of elements.
+        """Selects N number of elements (given as locators) in the order they are given. This is
+        useful for working though a nested menu listing of elements.
 
         On failure this keyword will attempt to select the elements from the desktop session due to
         the nature of some pop-out menus in Windows."""
@@ -741,8 +761,9 @@ class DesktopLibrary(AppiumLibrary):
 
     @keyword("Select Elements From Context Menu")
     def select_elements_from_context_menu(self, *args):
-        """Context clicks the first element and then selects N number of elements in the order they
-        are given. This is useful for working though a nested context menu listing of elements.
+        """Context clicks the first element and then selects N number of elements (given as
+        locators) in the order they are given. This is useful for working though a nested context
+        menu listing of elements.
 
         On failure this keyword will attempt to select the elements from the desktop session due to
         the nature of some pop-out menus in Windows."""
@@ -799,7 +820,8 @@ class DesktopLibrary(AppiumLibrary):
     @keyword("Double Tap")
     def double_tap(self, locator):
         """ Double tap element identified by ``locator``."""
-        element = self._element_find(locator, True, True)
+        element = self._check_for_cached_element(locator)
+        self._info("Double Tapping on locator %s." % locator)
         action = TouchActions(self._current_application())
         action.double_tap(element).perform()
 
@@ -813,7 +835,9 @@ class DesktopLibrary(AppiumLibrary):
 
         See `introduction` for details about locating elements."""
         self._wait_until_page_contains_element(locator, timeout, error)
-        self.tap(locator)
+        self._info("Tapping on locator %s." % locator)
+        action = TouchAction(self._current_application())
+        action.tap(self.current_element).perform()
 
     @keyword("Wait For And Double Tap")
     def wait_for_and_double_tap(self, locator, timeout=None, error=None):
@@ -825,7 +849,7 @@ class DesktopLibrary(AppiumLibrary):
 
         See `introduction` for details about locating elements."""
         self._wait_until_page_contains_element(locator, timeout, error)
-        self.double_tap(locator)
+        self.double_tap(self.current_element)
 
     @keyword("Flick")
     def flick(self, x_speed, y_speed):
@@ -846,7 +870,7 @@ class DesktopLibrary(AppiumLibrary):
         ``y_offset`` is Y offset to flick to.
 
         ``speed`` is Pixels per second to flick."""
-        element = self._element_find(locator, True, True)
+        element = self._check_for_cached_element(locator)
         action = TouchActions(self._current_application())
         action.flick_element(element, x_offset, y_offset, speed).perform()
 
@@ -861,7 +885,7 @@ class DesktopLibrary(AppiumLibrary):
 
         See `introduction` for details about locating elements."""
         self._wait_until_page_contains_element(locator, timeout, error)
-        self.flick_from_element(locator, x_offset, y_offset, speed)
+        self.flick_from_element(self.current_element, x_offset, y_offset, speed)
 
     @keyword("Scroll")
     def scroll(self, x_offset, y_offset):
@@ -880,7 +904,7 @@ class DesktopLibrary(AppiumLibrary):
          ``x_offset`` is the X offset to scroll to.
 
          ``y_offset`` is the Y offset to scroll to."""
-        element = self._element_find(locator, True, True)
+        element = self._check_for_cached_element(locator)
         action = TouchActions(self._current_application())
         action.scroll_from_element(element, x_offset, y_offset).perform()
 
@@ -895,7 +919,7 @@ class DesktopLibrary(AppiumLibrary):
 
         See `introduction` for details about locating elements."""
         self._wait_until_page_contains_element(locator, timeout, error)
-        self.scroll_from_element(locator, x_offset, y_offset)
+        self.scroll_from_element(self.current_element, x_offset, y_offset)
 
     # Private
     def _move_to_element(self, actions, element, x_offset=0, y_offset=0):
@@ -959,22 +983,27 @@ class DesktopLibrary(AppiumLibrary):
         driver = self._current_application()
         if prefix is None:
             if criteria.startswith('//'):
-                return len(driver.find_elements_by_xpath(criteria)) > 0
-            return len(driver.find_elements_by_accessibility_id(criteria)) > 0
-        if prefix == 'name':
-            return len(driver.find_elements_by_name(criteria)) > 0
-        if prefix == 'class':
-            return len(driver.find_elements_by_class_name(criteria)) > 0
-        if prefix == 'xpath':
-            return len(driver.find_elements_by_xpath(criteria)) > 0
-        if prefix == 'accessibility_id':
-            return len(driver.find_elements_by_accessibility_id(criteria)) > 0
-        if prefix == 'image':
+                elements_list = driver.find_elements_by_xpath(criteria)
+            else:
+                elements_list = driver.find_elements_by_accessibility_id(criteria)
+        elif prefix == 'name':
+            elements_list = driver.find_elements_by_name(criteria)
+        elif prefix == 'class':
+            elements_list = driver.find_elements_by_class_name(criteria)
+        elif prefix == 'xpath':
+            elements_list = driver.find_elements_by_xpath(criteria)
+        elif prefix == 'accessibility_id':
+            elements_list = driver.find_elements_by_accessibility_id(criteria)
+        elif prefix == 'image':
             try:
-                return len(driver.find_elements_by_image(criteria)) > 0
+                elements_list = driver.find_elements_by_image(criteria)
             except InvalidSelectorException:
                 zoomba.fail("Selecting by image is only available when using Appium v1.18.0 or higher")
-        zoomba.fail("Element locator with prefix '" + prefix + "' is not supported")
+        else:
+            zoomba.fail("Element locator with prefix '" + prefix + "' is not supported")
+        if len(elements_list) > 0:
+            self.current_element = elements_list[0]
+        return len(elements_list) > 0
 
     def _parse_locator(self, locator):
         prefix = None
@@ -1007,3 +1036,14 @@ class DesktopLibrary(AppiumLibrary):
             if time() > max_time:
                 raise AssertionError(timeout_error)
             sleep(0.2)
+
+    def _get_text(self, locator):
+        element = self._check_for_cached_element(locator)
+        if element is not None:
+            return element.text
+        return None
+
+    def _check_for_cached_element(self, locator):
+        if locator == self.current_element:
+            return self.current_element
+        return self._element_find(locator, True, True)
